<?php

namespace TheBugs\Email;

class SwiftMailer implements MailerInterface
{
    use EmailInfoTrait;
    
    protected $transport;
    
    public function __construct(EmailConfiguration $configuration) {
        $this->emailConfiguration = $configuration;
        
        $this->transport = ( new \Swift_SmtpTransport($this->emailConfiguration->smtpHost, $this->emailConfiguration->smtpPort) )
            ->setUsername($this->emailConfiguration->smtpUsername)
            ->setPassword($this->emailConfiguration->smtpPassword);
    }
    
    public function send(string $subject, string $message, bool $html = true) : bool
    {
        $swiftObj = ( new \Swift_Message($subject) )
            ->setFrom( $this->from )
            ->setTo( $this->to )
            ->setCC($this->cc ?? [])
            ->setBCC($this->bcc ?? [])
            ->setBody( $message, $html ? 'text/html' : 'text/plain' );

        foreach ($this->attachments as $attachment) {
            $swiftObj->attach($attachment);
        }

        $this->attachments = [];

        return ( new \Swift_Mailer($this->transport) )->send($swiftObj);
    }

    public function attach(mixed $attachment) : self
    {
        $this->attachments[] = $attachment;

        return $this;
    }
}