<?php

namespace TheBugs\Email;

use Symfony\Component\Mailer\{ Mailer, Transport };
use Symfony\Component\Mime\Email;

class SymfonyMailer implements MailerInterface
{
    use EmailInfoTrait;

    public Mailer $mailer;

    public function __construct(EmailConfiguration $configuration) {
        $this->emailConfiguration = $configuration;

        if ($this->emailConfiguration->smtpUsername) {
            $this->mailer = new Mailer(Transport::fromDsn(sprintf("smtp://%s:%s@%s:%d?verify_peer=%s", $this->emailConfiguration->smtpUsername, $this->emailConfiguration->smtpPassword, $this->emailConfiguration->smtpHost, $this->emailConfiguration->smtpPort, $this->emailConfiguration->smtpVerifyPeer ? 'true' : 'false')));
        }
        else {
            $this->mailer = new Mailer(Transport::fromDsn(sprintf("smtp://%s:%d?verify_peer=%s", $this->emailConfiguration->smtpHost, $this->emailConfiguration->smtpPort, $this->emailConfiguration->smtpVerifyPeer ? 'true' : 'false')));
        }
    }

    public function send(string $subject, string $message, bool $html = true) : bool
    {
        $email = ( new Email() )
            ->subject($subject)
            ->from(...$this->from())
            ->to(...$this->to())
            ->cc(...$this->cc())
            ->bcc(...$this->bcc());

        if ($html) {
            $email->html($message);
        }
        else {
            $email->text($message);
        }

        foreach ($this->attachments as $path) {
            $email->attachFromPath($path);
        }

        $this->attachments = [];

        try {
            $this->mailer->send($email);
        }
        catch(\Exception $e) {
            return false;
        }

        return true;
    }

    public function attach(mixed $attachment) : self
    {
        $this->attachments[] = $attachment;

        return $this;
    }

    protected function from() : array
    {
        return $this->arrayToAddress($this->from);
    }

    protected function to() : array
    {
        return $this->arrayToAddress($this->to);
    }

    protected function cc() : array
    {
        return $this->arrayToAddress($this->cc);
    }

    protected function bcc() : array
    {
        return $this->arrayToAddress($this->bcc);
    }

    protected function arrayToAddress(string|\Stringable|array $array) : array
    {
        $list = [];

        foreach((array) $array as $addr => $name) {
            $list[] = "$name <$addr>";
        }

        return $list;
    }
}